; Copyright (c) 1993, 1994 Richard Kelsey and Jonathan Rees.  See file COPYING.


; Using sockets from Scheme 48.

; Server.  To make your program be a server, do
;  (call-with-values socket-server
;    (lambda (port-number accept)
;      (display "Port number is ") (write port-number) (newline)
;      (call-with-values accept
;        (lambda (i-port o-port)
;          ... have fun ...))))

(define (socket-server)
  (let* ((socket (internet-stream-socket))
	 (channel (open-channel socket (enum open-channel-option
					     special-input-channel))))
    (values (socket-bind socket)
	    (lambda ()
	      (fd->ports (socket-accept channel))))))

; Client.  To make your program be a client, do
;  (call-with-values (lambda ()
;                      (socket-client "foo.mit.edu" port-number))
;    (lambda (i-port o-port)
;      ... have fun ...))

(define (socket-client host-name port-number)
  (fd->ports (socket-connect host-name port-number)))


; vm-extension calls

(define (internet-stream-socket)
  (vm-extension 20 #f))

(define (socket-bind s)
  (vm-extension 21 s))

(define (socket-accept socket-channel)
  (let ((fd (vm-extension 22 socket-channel)))
    (cond ((= fd -1)  ; not ready
	   (block-on-channel socket-channel)
	   (socket-accept socket-channel))
	  (else
	   fd))))

(define (socket-connect host-name port-number)
  (vm-extension 23 (cons host-name port-number)))

; We duplicate the fd so that the input and output channels can be
; closed independently.

(define (fd->ports fd)
  (let ((out-fd (duplicate-fd fd)))
    (values (input-channel->port (open-channel fd (enum open-channel-option
							raw-input-channel))
				 1024)  ; buffer size
	    (output-channel->port (open-channel out-fd (enum open-channel-option
							     raw-output-channel))
				  1024))))

(define (duplicate-fd fd)
  (vm-extension 29 fd))
