
; Stuff in Pre-Scheme that is not in Scheme.

(define (ps-read-char port char-cont eof-cont)
  (let ((next (read-char port)))
    (if (eof-object? next)
	(eof-cont)
	(char-cont next))))

(define ashl arithmetic-shift)

(define (ashr i n)
  (arithmetic-shift i (- 0 n)))

(define small* *) ; could do a range check

; What we will get in C.
(define pre-scheme-integer-size 32)

(define int-mask (- (arithmetic-shift 1 pre-scheme-integer-size) 1))

(define (lshr i n)
  (if (>= i 0)
      (arithmetic-shift i (- 0 n))
      (arithmetic-shift (bitwise-and i int-mask) (- 0 n))))

(define (min x y)
  (if (<= x y) x y))

(define (max x y)
  (if (<= x y) x y))

(define (deallocate x) #f)

(define unassigned unspecific)

(define (write-string str port)
  (do ((i 0 (+ i 1)))
      ((>= i (string-length str)))
    (write-char (string-ref str i) port)))

(define-syntax goto
  (lambda (exp rename compare)
    (cdr exp)))

(define (null-port? x)
  #f) ; no null-ports in Scheme

(define (call-external-value proc nargs pointer-to-args)
  (error "~S" (list 'call-external-value proc nargs pointer-to-args)))

(define-syntax external
  (lambda (exp rename compare)
    (error "(EXTERNAL ...) can only be compiled" exp)))

(define (make-port-set)
  (vector '()))

(define (clear-port-set! set)
  (vector-set! set 0 '()))

(define (add-to-port-set! set port)
  (if (not (memq port (vector-ref set 0)))
      (vector-set! set 0 (cons port (vector-ref set 0)))))

(define (port-set-member? set port)
  (memq port (vector-ref set 0)))

; No other way in Scheme.

(define (ready-for-writing? port)
  #t)

; Output should be empty because output ports never block.

(define (find-ready-ports waiting-for-input waiting-for-output count block?)
  (let ((in (vector-ref waiting-for-input 0)))
    (let loop ()
      (let ((ready (do ((ps (vector-ref waiting-for-input 0) (cdr ps))
			(res '() (if (char-ready? (car ps))
				     (cons (car ps) res)
				     res)))
		       ((null? ps)
			res))))
	(cond ((not (null? ready))
	       (vector-set! waiting-for-input 0 ready)
	       (length ready))
	      (block?
	       (loop))   ; Yow! Busy waiting!
	      (else
	       0))))))

(define (coerce thing from to) thing)

(define current-error-port current-output-port)